--------------------------------------------------------
-- Schritt 1:  FileStream auf Dateiebene aktivieren 
--             ber Konfigurationsmanager 


--------------------------------------------------------
-- Schritt 2: FileStream auf Instanzebene aktivieren 
-- 0 - Disables FILESTREAM support for this instance.
-- 1 - Enables FILESTREAM for Transact-SQL access.
-- 2 - Enables FILESTREAM for Transact-SQL and Win32 streaming access.
EXEC sp_configure filestream_access_level, 2 RECONFIGURE


--- Mit "Net Share" die interne Freigabe des SQL Servers einsehen 


--------------------------------------------------------
-- Schritt 3: Datenbank anlegen
-- Voraussetzung: Unter X: muss der Ordner "KuechenCurioDB_Dateien" existieren
USE [master]
GO
-- Drop Database KuechenCurioDB
CREATE DATABASE [KuechenCurioDB]
on primary
( NAME = N'KuechenCurioDB', FILENAME = N'M:\KuechenCurioDB.mdf' , SIZE = 5120KB, MAXSIZE = UNLIMITED, FILEGROWTH = 1024KB ), 
 FILEGROUP [KuechenCurio_DG_FS] CONTAINS FILESTREAM  DEFAULT 
( NAME = N'KuechenCurioDB_FS_Ordner', FILENAME = N'X:\KuechenCurioDB_Dateien\KuechenCurioDB_FS_Ordner' , MAXSIZE = UNLIMITED)
 LOG ON 
( NAME = N'KuechenCurioDB_log', FILENAME = N'O:\KuechenCurioDB_log.ldf' , SIZE = 1024KB , MAXSIZE = 2048GB , FILEGROWTH = 10%)
GO


-- In der GUI das Ergebnis zeigen. 


-- Filestream-Angabe  ist "nur" ein Ordner. 
-- Ein leerer Ordner hat keine Anfangsgre. Wie auch? 
-- Wer mchte den Ordner sehen? 
-- X:\KuechenCurioDB_Dateien\KuechenCurioDB_FS_Ordner



--------------------------------------------------------
-- Schritt 4: Tabelle mit Binr-Daten 

use KuechenCurioDB 
go 
-- drop table Artikel 
CREATE TABLE [dbo].[Artikel]  
(
	[ID_Artikel] [int] IDENTITY PRIMARY KEY NOT NULL,
	[Artikel_Name] [nvarchar](50) NULL,
	[Artikel_Preis] [decimal](12, 3) NULL, 
	[Artikel_Dokument] [varbinary](max) FileStream NULL, 
	[Artikel_Dokument_ID]  [UNIQUEIDENTIFIER]  ROWGUIDCOL UNIQUE  DEFAULT (newid())  NOT NULL  
 ) Filestream_on KuechenCurio_DG_FS



 ---------------------------------------------------------
 -- Schritt 5a: Einfgen eines Artikels mit Beschreibung 
Insert into Artikel
	(Artikel_Name, Artikel_Preis, Artikel_Dokument)
values
	('Herd', 550, cast('Das sind die Binrdaten des Dokuments' as varbinary(max)) ) 

-- Abfragen 
Select * from Artikel
 
-- Zur Kontrolle schaue man unter X:\KuechenCurioDB_Dateien


--   Daten                            Dateien
--     |                                 |
--    \|/				\|/
-- tbl_Artikel               |    KuechenCurioDG_FS
-----------------------------|-------------------------------
-- Artikel_Name              |    Artikel_Beschreibung 
-- Artikel_Preis             |    
-- Artikel_Beschreibung_ID   |


-- FileStream-Datei zwar im Dateisystem, aber ... 
-- ... der SQL Server wei jetzt von der Datei! 
	SELECT Artikel_Name, Artikel_Dokument_ID, Artikel_Dokument.PathName() as PfadAngabe FROM Artikel


----------------------------------------------------------
-- Schritt 5b: Einfgen eines Artikels mit Dokument 
Insert into Artikel
	(Artikel_Name, Artikel_Preis, Artikel_Dokument)
Select 
	  'Sple'
	, 150
	, * FROM OPENROWSET(BULK N'X:\FileStream_Vortrag\1_Beschreibung_1Sple.txt', SINGLE_BLOB) AS Artikel_Dokument;
GO

-- Abfragen
Select * from artikel 

-- Zur Kontrolle schaue man unter X:\KuechenCurioDB_Dateien
-- Das "pdf" kopieren und ffnen. 
-- DAS NIE IN EINEM PRODUKTIVEN Sytem tun!!! 
-- Das ist ein interner Ordner des SQL Servers!!!


----------------------------------------------------------
-- Schritt 5c: Einfgen eines Artikels mit riesem Dokument 
Insert into Artikel
	(Artikel_Name, Artikel_Preis, Artikel_Dokument)
Select 
	  'Schrank'
	, 250
	, * FROM OPENROWSET(BULK N'X:\FileStream_Vortrag\1_Beschreibung_2Schrank.pdf', SINGLE_BLOB) AS Artikel_Dokument;
GO

-- Abfragen
Select * from artikel 

-- Select *     war schon immer keine gute Idee, aber hier ist es ganz schlimm, ... 
--              ... weil die Dateien jeweils geladen werden. 


-- Das dauert etwas, weil der SQL Server die 114 MB in einlesen muss, ...
-- ... um sie in den Filestream-Ordner zu speichern. 
-- FILESTREAM verwendet den NT-Systemcache zum Zwischenspeichern von Dateidaten. 
-- Dies wirkt allen negativen Auswirkungen entgegen, die FILESTREAM-Daten 
-- auf die Database Engine (Datenbankmodul)-Leistung haben knnten. 
-- Der SQL Server-Pufferpool wird nicht verwendet, deshalb steht dieser Arbeitsspeicher 
-- fr die Abfragebearbeitung zur Verfgung.
-- Quelle: http://technet.microsoft.com/de-de/library/gg471497.aspx -> 3. Absatz



-- Warum geht die Datei nicht ber das Log? 
-- Weil nur genderte Pages im Log stehen. Aber die Datei an sich nicht. 



----------------------------------------------------------
-- Schritt 5d: Einfgen mehrerer Artikel 
--             und Transaktion! 
Begin Tran Kein_Insert
Insert into Artikel
	(Artikel_Name, Artikel_Preis, Artikel_Dokument)
Select 
	  'Besteck'
	, 25
	, * FROM OPENROWSET(BULK N'X:\FileStream_Vortrag\1_Beschreibung_3Besteck.txt', SINGLE_BLOB) AS Artikel_Dokument;
Insert into Artikel
	(Artikel_Name, Artikel_Preis, Artikel_Dokument)
Select 
	  'Ofen'
	, 850
	, * FROM OPENROWSET(BULK N'X:\FileStream_Vortrag\1_Beschreibung_4Ofen.txt', SINGLE_BLOB) AS Artikel_Dokument;
Rollback 


-- Abfragen
Select * from artikel 


-- Der Garbarge-Collector rumt irgendwann auf. 
-- Dateien sind verfgbar wegen z.B. Always on, um nachzuvollziehen, was da wann passiert war. 

-- Beispielanwendung: 
-- Mitarbeiter bekommen Mails mit Anhngen im CRM-System. 
-- Die Programmierung soll die Mails speichern ... 
-- Dann per .Net den Anhang in die Freigabe "schicken". 