-- Unterschied FileStream und FileTable
-- Wenn ein Anwneder eine Mail mit Anhang bekommt, dann soll meine Programmierung den Anhang ... 
-- ... automatisch aus dem CRM-System abholen --> Filestream. 
-- --> Programmierung soll holen, dann FileStream
-- Wenn ein Dokument automatisch eingescannt wird und anschlieend in ein ... 
-- ... Verzeichnis gelegt wird, dann aktivieren wir FileTable, weil dann der Rest ... 
-- ... "Automatisch" geht. Gerade hier, weil die Dokumente aus dem Scanvorgang ... 
-- ... einheitliche Namen bekommen, so dass eine Zuordnung mglich ist. 


-- Erstellen einer Mitarbeiter-Tabelle 
Create table Mitarbeiter 
	( ID_Mitarbeiter [bigint] IDENTITY(1,1)  not null 
	, Nachname nvarchar(50)
	, Vorname nvarchar(50)
	, Strasse nvarchar(50)
	, PLZ nvarchar(20)
	, Ort nvarchar(50)
	, fi_Bewerbung_ID uniqueidentifier) 
Go 

-- ein paar Daten 
Insert into Mitarbeiter
	(Nachname, Vorname, Strasse, PLZ, Ort) 
values
	  ('Curio', 'John', 'Friedrichstr. 56', '50676', 'Kln' )
	, ('Gro', 'Steffi', 'Palmentstr. 28', '4444', 'Basel' )
	, ('Schulze', 'Martin', 'Kleestr. 9', '10111', 'Berlin' )



-- Erstellen der Tabelle 
Create Trigger trg_Insert
	on Produktbeschreibung
	after insert
as
	begin
	Declare @unterlagen uniqueidentifier
	Select @unterlagen = [stream_id] from inserted 
	update Mitarbeiter 
	set fi_Bewerbung_ID = (@unterlagen) 
	where Nachname = 'curio'
end 


-- Datei "Bewerbung Curio" in den Ordner verschieben 

-- Abfrage komplett 
select * from Mitarbeiter
select * from Produktbeschreibung

-- Join 
select m.Nachname, f.name as Dateiname, f.cached_file_size as Size
	from mitarbeiter m 
inner join Produktbeschreibung f 
	on m.fi_Bewerbung_ID = f.stream_id 


-- Fazit: 
-- FileTable ist eine normale Tabelle, nur ihre Struktur kann nicht gendert werden. 

-----------------------------------------------
-- Andere Idee: 
-- Versionierung der Dokumente meines Dokumentenmanagement-System 
-- ber Trigger eine Sicherheitskopie erstellen, ... 
-- ... um Versionierung darzustellen.   
-- ber Agent: Alle Versionen lter als 12 Jahre lschen. 


-------------------------------------------------------
-- Ersatz fr FileServer? 
-- Das ist aber kein Ersatz fr den Dateiserver, weil ... 
-- ... keine atomare Berechtigungsstruktur. 
-- SQL Server KEINE Berechtigung auf Zeilenebene 
-- Alle Dateien sind "gleichberechtigt".  

Select name, p.creation_time, p.last_access_time, p.last_write_time from produktbeschreibung p

-- User Franz auf der Datenbank 
	CREATE USER [Franz] FOR LOGIN [KUECHENCURIO\Franz]
-- User Franz darf die Ordnerstuktur sehen, aber nicht die Dateien
	GRANT VIEW DEFINITION ON [dbo].[Produktbeschreibung] TO [Franz]
-- User Franz darf die 
	GRANT SELECT ON [Produktbeschreibung] TO [Franz]


--> Franz hat Rechte auf ALLE (Unter-)Ordner und Dateien innerhalb der Produktbeschreibung


-----------------------------------------------
-- Datenbanken sichern und wiederherstellen 
-- 0. Datenbank sichern 
-- 1. Datenbank lschen 
-- 2. Dateien lschen
-- 3. Wiederherstellen! Alles wieder da! 

-- Wichtig: Rcksichern einer einzelen Datei ... geht nicht! 
-- Wichtig: Die Dateien mssen nicht mehr aus dem Fileserver gesichert werden. 
--			Die liegen jetzt unter der Kontrolle der Datenbank!
